///////////////////////////////////////////////////////////////////////////////
//
//	File    : $Id: exceptionhelper.cpp,v 1.8.2.2 2004/09/23 15:34:07 epocman Exp $
//	Subject : IBPP, ExceptionHelper implementation
//
///////////////////////////////////////////////////////////////////////////////
//
//	The contents of this file are subject to the Mozilla Public License
//	Version 1.0 (the "License"); you may not use this file except in
//	compliance with the License. You may obtain a copy of the License at
//	http://www.mozilla.org/MPL/
//
//	Software distributed under the License is distributed on an "AS IS"
//	basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
//	License for the specific language governing rights and limitations
//	under the License.
//
//	The Original Code is "IBPP 0.9" and all its associated documentation.
//
//	The Initial Developer of the Original Code is T.I.P. Group S.A.
//	Portions created by T.I.P. Group S.A. are
//	Copyright (C) 2000 T.I.P Group S.A.
//	All Rights Reserved.
//
//	Contributor(s): ______________________________________.
//
///////////////////////////////////////////////////////////////////////////////
//
//	COMMENTS
//	* ExceptionHelper is a C++ exception class. It is always thrown by reference
//	  from inside the implementation of IBPP. It is always catched by reference
//	  as an IBPP::Exception class in code that use the IBPP library. The 'user'
//	  code does not have the ability to throw that exception.
//	* Tabulations should be set every four characters when editing this file.
//
///////////////////////////////////////////////////////////////////////////////

#include "ibpp.h"
#include "_internals.h"

#ifdef HAS_HDRSTOP
#pragma hdrstop
#endif

#include <stdarg.h>
#include <stdio.h>

using namespace ibpp_internals;

//	(((((((( OBJECT INTERNAL METHODS ))))))))

void ExceptionHelper::buildErrorMessage(const char* message)
{
	mErrorMessage.assign("*** IBPP::Exception ***\n");
	if (! mContext.empty())
		mErrorMessage.append("Context: ").append(mContext).append("\n");

	if (message != 0 && *message != 0 )
		mErrorMessage.append("Message: ").append(message).append("\n");
	
	mErrorMessage.append("\n");
}

ExceptionHelper::ExceptionHelper(const ExceptionHelper& copied)
{
	mContext = copied.mContext;
	mErrorMessage = copied.mErrorMessage;
}

ExceptionHelper& ExceptionHelper::operator=(const ExceptionHelper& copied)
{
	//	The operator= is VERY important for a class used
	//	as a C++ exception, especially with Borland BCC32 5.5.

	mContext = copied.mContext;
	mErrorMessage = copied.mErrorMessage;
	return *this;
}

ExceptionHelper::ExceptionHelper(const char* context, const char* message, ...)
{
	if (context == 0) mContext.erase();
	else mContext.assign(context);

	if (message != 0)
	{
		char buffer[1024];
		va_list argptr;
		va_start(argptr, message);
#if defined(IBPP_MSVC) || defined(IBPP_DMC)
		_vsnprintf(buffer, sizeof(buffer)-1, message, argptr);
#else
		vsnprintf(buffer, sizeof(buffer)-1, message, argptr);
#endif
		buffer[sizeof(buffer)-1] = 0;
		va_end(argptr);
	
		buildErrorMessage(buffer);
	}
	else
		buildErrorMessage(0);
}

//
//	EOF
//
